/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

///////////////////////////////////////////////////////////////////////////
// Filename	: TciGrabber.cpp
// Created	: Tuesday, June 17th, 1997
// Description: Implements a class to encapsulate the functionality of the Coreco TCI
//					board.
// 
//  Vasanth Philomin 1997, vasi@cs.umd.edu
///////////////////////////////////////////////////////////////////////////
// TciGrabber.cpp : implementation file
//

#include "stdafx.h"
#include "TciGrabber.h"
#include "VTimer.h"

static ODXPARM OldAcqAtt;
static CRITICAL_SECTION g_csBuf;

VTciGrabber::VTciGrabber():stopEvent(FALSE, FALSE, "Grabber Stop Event")
{
	
}

VTciGrabber::~VTciGrabber()
{
	StopGrab();
}

BOOL VTciGrabber::Init(CStringList *msgs, int res_factor)
{
	
	int		nodx=0;
	// Do odx binding
	// Bind to a driver
	if ( (nodx = odxbind() ) < 1)	{
		// Try to install the driver
		system( "odtci");
		if ( (nodx = odxbind() ) < 1 ) {
			AfxMessageBox("Unable to bind to a driver", NULL, MB_OK | MB_ICONHAND);
			return FALSE;
		}
	}
	int	err = opr_inq(ERRNO);
	if (  err != INOERR ) {
		// Try to install the driver
		if ( system( "odtci") == (-1) ) {
			AfxMessageBox("Unable to install the Coreco driver", NULL, MB_OK | MB_ICONHAND);
			return FALSE;
		}
		if ( (nodx = odxbind() ) < 1 ) {
			AfxMessageBox("Unable to bind to a driver", NULL, MB_OK | MB_ICONHAND);
			return FALSE;
		}
	}
	
	// Get the driver ID
	char	str_driverID[30];
	int		dID = 0;
	dID = odxid( str_driverID );
	
	setdmajor(0);
	
	int nVgaPlanes, nVgaBitsPerPixel;
	// Get the number of VGA planes and bits per pixel
	CDC *pDC = AfxGetMainWnd()->GetDC();
	ASSERT(pDC);
	nVgaPlanes = pDC->GetDeviceCaps(PLANES);
	nVgaBitsPerPixel = pDC->GetDeviceCaps(BITSPIXEL);
	AfxGetMainWnd()->ReleaseDC(pDC);
	// Print values to the message box
	if (msgs)
		msgs->AddTail("VGA:");
	CString s;
	s.Format("Planes, Bits per Pixel = %d, %d", nVgaPlanes, nVgaBitsPerPixel);
	if (msgs)
		msgs->AddTail(s);
	// Select the host frame buffer
	select_minor(HOST_FRAME_BUFFER);
	// Get the Host Frame buffer pixel size
	m_nPixSize = opr_inq(PIXSIZ);
	
	// Get the Host Frame buffer dimensions
	int nWidth  = opr_inq(MW);
	int nHeight = opr_inq(MH);
	
	// Get the hue/saturation/brightness/contrast information
	s.Format("Acquisition saturation %d, brightness %d, %d, %d, hue %d, contrast %d, %d, %d, range %d, %d, %d\n",
				    opr_inq(AIC_SATURATION), 
					opr_inq(AIC_BRIGHTNESS_RED),opr_inq(AIC_BRIGHTNESS_GREEN),opr_inq(AIC_BRIGHTNESS_BLUE),
					opr_inq(AIC_HUE), 
					opr_inq(AIC_CONTRAST_RED),opr_inq(AIC_CONTRAST_GREEN),opr_inq(AIC_CONTRAST_BLUE),
					opr_inq(AIC_RANGE_RED),opr_inq(AIC_RANGE_GREEN),opr_inq(AIC_RANGE_BLUE) );
	//	opr_set( AIC_SATURATION, 125 );
	opr_set(AIC_BRIGHTNESS_RED, 148);
	opr_set(AIC_BRIGHTNESS_GREEN, 148);
	opr_set(AIC_BRIGHTNESS_BLUE, 148);
	//	opr_set(AIC_HUE, 125);
	opr_set(AIC_CONTRAST_RED,124);
	opr_set(AIC_CONTRAST_GREEN,124);
	opr_set(AIC_CONTRAST_BLUE,124);
	opr_set(AIC_RANGE_RED,255);
	opr_set(AIC_RANGE_GREEN,255);
	opr_set(AIC_RANGE_BLUE,255);
	s.Format("Acquisition saturation %d, brightness %d, %d, %d, hue %d, contrast %d, %d, %d, range %d, %d, %d\n",
				    opr_inq(AIC_SATURATION), 
					opr_inq(AIC_BRIGHTNESS_RED),opr_inq(AIC_BRIGHTNESS_GREEN),opr_inq(AIC_BRIGHTNESS_BLUE),
					opr_inq(AIC_HUE), 
					opr_inq(AIC_CONTRAST_RED),opr_inq(AIC_CONTRAST_GREEN),opr_inq(AIC_CONTRAST_BLUE),
					opr_inq(AIC_RANGE_RED),opr_inq(AIC_RANGE_GREEN),opr_inq(AIC_RANGE_BLUE) );
	s.Format("Acquisition offset %d, %d, %d\n",
		opr_inq(AIC_OFFSET_RED),opr_inq(AIC_OFFSET_GREEN),opr_inq(AIC_OFFSET_BLUE) );
	
	
	// Get type of info. stored in each buffer
	int fbtype = opr_inq(FBTYPE);
	
	// Acquire half scale
	m_nWidth = nWidth/res_factor;
	m_nHeight = nHeight/res_factor;
	//acqscale(nWidth, nHeight);  // Acquire full scale
	// Set the acquisition scale - Note that fwin gets set automatically
	acqscale(m_nWidth, m_nHeight);
	
	// Write these values to the message box
	if (msgs)
		msgs->AddTail("HOST FRAME BUFFER:");
	s.Format("Pixel size = %d", m_nPixSize);
	if (msgs)
		msgs->AddTail(s);
	s.Format("Width, Height = %d, %d", nWidth, nHeight);
	if (msgs)
		msgs->AddTail(s);
	char buf[10];
	s.Format("Frame buffer type = %s", (fbtype == FB_MONO ? "FB_MONO" : (fbtype == FB_RGBPACK ?
		"FB_RGBPACK" : (fbtype == FB_RGBMULTI ? "FB_RGBMULTI" : _itoa(fbtype, buf, 10)))));
	if (msgs)
		msgs->AddTail(s);
	int nFrameBuffers = (int)opr_inq(FBCNT);
	s.Format("Number of Frame Buffers = %d", nFrameBuffers);
	if (msgs)
		msgs->AddTail(s);
	for (int j=0;j<nFrameBuffers;j++)	{
		opr_set(FFB, j);
		acqscale(m_nWidth, m_nHeight);
		opr_set(PFB, j);
		// Set the Proc. window size equal to the fwin size
		pwin(opr_inq(FWIN_XMIN), opr_inq(FWIN_YMIN), opr_inq(FWIN_XLEN), opr_inq(FWIN_YLEN));
	}
	s.Format("PWIN size: (%d, %d, %d, %d)", opr_inq(PWIN_XMIN), opr_inq(PWIN_YMIN),
		opr_inq(PWIN_XLEN), opr_inq(PWIN_YLEN));
	if (msgs)
		msgs->AddTail(s);
	
	// Setup Image Source addresses from host frame buffers
	opr_set(PFB, 0);
	ImgBuf1 = (DWORD *) memopen(0);
	opr_set(PFB, 1);
	ImgBuf2 = (DWORD *) memopen(0);
	
	CYCLEDEF CycleParms;
	// Read the current Cycle definition parameters
	deffbcycle(0, &CycleParms);
	CycleParms.version       = CYCLEDEF_VERSION;
	CycleParms.fbstart       = 0;	// Use only the first two frame ...
	CycleParms.fbend         = 1;	// ... buffers on the host
	CycleParms.dx            = 0;	// Allow only one frame ...
	CycleParms.dy            = 0;	// ... per frame buffer
	CycleParms.flag          = 0;	// Not used
	
	// Initialize deffbcycle
	deffbcycle(1, &CycleParms);	// 1 => Write
	
	// Enable the fbcycle grab
	OldAcqAtt = opr_inq(ACQATTRIBUTE);
	opr_set(ACQATTRIBUTE, OldAcqAtt | ACQATTRIBUTE_FBCYCLE);
	
	m_bGrabbing = FALSE;
	
	// Initialize the critical section used to lock the buffer
	::InitializeCriticalSection(&g_csBuf);
	
	int bufSize = Width() * Height() * 4;
	
	hBuf = GlobalAlloc(GPTR, bufSize);
	VERIFY(hBuf);
	
	m_lpBits = GlobalLock(hBuf);
	
	return TRUE;
}

void VTciGrabber::Dispose()
{
	StopGrab();
	GlobalUnlock(hBuf);
	GlobalFree(hBuf);
	
	// Dispose the critical section used to lock the buffer
	::DeleteCriticalSection(&g_csBuf);
	
	// Restore the old acq. attribute
	opr_set(ACQATTRIBUTE, OldAcqAtt);
	// Close the memory buffers
	opr_set(PFB, 0);
	memclose(0);
	opr_set(PFB, 1);
	memclose(0);	
	odxunbind();         // Release the driver
}

void VTciGrabber::Snap(CDisplay *disp)
{
	ASSERT(disp);
	fbgrab(0);	// Stop any ongoing grab
	while (opr_inq(FGON));
	opr_set(FFB, 0);
	opr_set(PFB, 0);
	opr_set(FBCYCLE_INDEX, 0);
	fbgrab(1);	// Grab a frame
	while (opr_inq(FGON));
	opr_set(PFB, 0);
	
	int bufSize = m_nWidth * m_nHeight * 4;
	LPVOID lpBits;
	
	lpBits = (void *)new char[bufSize];
	ASSERT(lpBits);
	
	rdbuf((ODXPTR) lpBits, m_nWidth);
	
	// Set the m_bValidDibFlag to TRUE so that
	// it updates the window everytime it refreshes
	disp->AttachMemToDib(lpBits,TRUE);
	disp->Refresh();
}


UINT GrabThread(LPVOID param)
{
	ThreadParams *params = (ThreadParams *)param;
	CDisplay *disp = params->disp;
	CListBox *lbMsgs = params->lbMsgs;
	VTciGrabber *grabber = params->grabber;
	BOOL display = params->display;
	
	
#ifdef _EVENTSUPPORT
	// Setup the ODXIsr interrupt handler
	SetMsgWindowHandle(params->hParentWnd);
	opr_set(EVENTMODE, EVENTMODE_END_OF_FRAME | EVENTMODE_START_OF_FIELD);
	OdxInstallProfileHandler();
	seteventfct(OdxIsr);
#endif
	
	CString s;
	grabber->m_nFramesGrabbed = 0;
	grabber->m_timer.Start();
	
	while (TRUE)	{
		if (::WaitForSingleObject(grabber->stopEvent.m_hObject, 0) == WAIT_OBJECT_0)
			break;
		grabber->m_nFramesGrabbed++;
		//#if defined _DEBUG || defined _PRINT
#if defined _PRINT
		if (lbMsgs)	{
			s.Format("Index = %d", opr_inq(FBCYCLE_INDEX));
			lbMsgs->AddString(s);
			s.Format("ErrNo = %d", opr_inq(ERRNO));
			lbMsgs->AddString(s);
			s.Format("PFB before = %d", opr_inq(PFB));
			lbMsgs->AddString(s);
		}
#endif
		fbgrab(1);	// Grab a frame
		// Process the other frame
#ifdef SINGLE_PROC
		opr_set(PFB, (i%2) ? 0 : 1);
#endif
		//#if defined _DEBUG || defined _PRINT
#if defined _PRINT
		if (lbMsgs)	{
			s.Format("PFB after = %d", opr_inq(PFB));
			lbMsgs->AddString(s);
		}
#endif
		::EnterCriticalSection(&g_csBuf);
		rdbuf((ODXPTR) grabber->m_lpBits, grabber->Width());
		::LeaveCriticalSection(&g_csBuf);
		// The pixels are stored in BGRX format - Little Endian
		if (display)	{
			disp->AttachMemToDib(grabber->m_lpBits);
			disp->Refresh();
		}
		while (opr_inq(FGON));
	}
	
	grabber->m_timer.Stop();
	if (lbMsgs)	{
		s.Format("Frame rate to grab %d frames = %f fps",grabber->m_nFramesGrabbed, 
			1000*grabber->m_nFramesGrabbed/(float)grabber->m_timer.Time());
		int k = lbMsgs->AddString(s);
		lbMsgs->SetCurSel(k);
	}
	
#ifdef _EVENTSUPPORT
	if (lbMsgs)	{
		s.Format("Number of frames actually grabbed = %ld", nInterrupts);
		lbMsgs->AddString(s);
	}
#endif
	
	
#ifdef _EVENTSUPPORT
	// Stop receiving interrupts
	OdxRemoveProfileHandler();
	seteventfct(NULL);
#endif
	grabber->m_bGrabbing = FALSE;	
	
	return 0;
}

void VTciGrabber::StopGrab()
{
	if ( m_bGrabbing==FALSE )
		return;
	stopEvent.SetEvent();
}

ErrorNumber VTciGrabber::StartGrab(HWND hWnd, 
								   CListBox *lstBox, 
								   CDisplay *disp, 
								   BOOL display)
{
	ErrorNumber			err = INOERR;
	if (m_bGrabbing)
		return err;
	
	// Set the first FBCYCLE_INDEX
	opr_set(FFB, 0);
	opr_set(PFB, 0);
	opr_set(FBCYCLE_INDEX, 0);
	
	CString s;
	//#if defined _DEBUG || defined _PRINT
#if defined _PRINT
	if (lstBox)	{
		s.Format("ErrNo = %d", opr_inq(ERRNO));
		lstBox->AddString(s);
	}
#endif
	
	fbgrab(1);	// Grab a frame into ImgBuf1
	
	// Wait for the grab to end so that we are certain that there is data in 
	int		dFrameStatus = opr_inq(FGON);
	switch (dFrameStatus) {
	case FG_OFF:
		AfxMessageBox("Unable to grab: grab not active", NULL, MB_OK | MB_ICONHAND);
		return dFrameStatus;
	case FG_LOCK:
		AfxMessageBox("Unable to grab: continuous grab active", NULL, MB_OK | MB_ICONHAND);
		return dFrameStatus;	
	case FG_ON:
	case FG_RQST:
		while ( (dFrameStatus == FG_ON) || (dFrameStatus == FG_RQST)) {
			dFrameStatus = opr_inq(FGON);
		}
		break;
	}
	
	params.display = display;
	params.grabber = this;
	params.hParentWnd = hWnd;
	params.disp = disp;
	params.lbMsgs = lstBox;
	
	CWinThread  	*pTh = 
		AfxBeginThread(GrabThread, &params, THREAD_PRIORITY_TIME_CRITICAL);
	
	m_Thread = pTh->m_hThread;
	m_bGrabbing = TRUE;
	return err;
}

ErrorNumber VTciGrabber::DirtyReduce(TBitMap &dest, int factor)
{
	if  ( !m_bGrabbing )
		return kErrNotGrabbing;
	
	CRect dRect = dest.GetValidRect();
	
	//#if defined _DEBUG || defined _PRINT
#if defined _PRINT
	if (m_nPixSize != dest.PixelSize())
		return kErrCantHandleEncoding;
	
	if ((dRect.Width() != Width()/factor) || (dRect.Height() != Height()/factor))
		return kErrWrongSize;
#endif
	
	LPBYTE fptr, dptr;		// Beginning of line pointers
	LPBYTE fpptr, dpptr;    // Pixel pointers
	int pixelSize = dest.PixelSize()/8;	// Pixel size in bytes
	int dbyteOffs = dest.LineOffset();
	int fbyteOffs = Width()*pixelSize*factor;
	int step = pixelSize*factor;
	
	// Calculate the base of the bitmap
	dptr = dest.GetBuffer() + abs(dRect.TopLeft().y)  * dbyteOffs + dRect.TopLeft().x * pixelSize;
	
	::EnterCriticalSection(&g_csBuf);
	
	// Get pointer to grabber memory
	fptr = (LPBYTE)m_lpBits;
	
	int x, y;
	switch (pixelSize)
	{
    case 1:
		for (y = Height() / factor; y > 0; y--)
		{
			fpptr = fptr;
			dpptr = dptr;
			
			for (x = Width() / factor; x > 0; x--)
			{
				*dpptr = *fpptr;
				fpptr += step;
				dpptr += pixelSize;
			}
			
			fptr += fbyteOffs;
			dptr += dbyteOffs;
		}
		break;
		
    case 2:
		for (y = Height() / factor; y > 0; y--)
		{
			fpptr = fptr;
			dpptr = dptr;
			
			for (x = Width() / factor; x > 0; x--)
			{
				*((LPWORD)dpptr) = *((LPWORD)fpptr);
				fpptr += step;
				dpptr += pixelSize;
			}
			
			fptr += fbyteOffs;
			dptr += dbyteOffs;
		}
		break;
		
    case 4:
		for (y = Height() / factor; y > 0; y--)
		{
			fpptr = fptr;
			dpptr = dptr;
			
			for (x = Width() / factor; x > 0; x--)
			{
				*((LPDWORD)dpptr) = *((LPDWORD)fpptr);
				fpptr += step;
				dpptr += pixelSize;
			}
			
			fptr += fbyteOffs;
			dptr += dbyteOffs;
		}
		break;
	}
	
	
	::LeaveCriticalSection(&g_csBuf);
	
	return kErrNoErr;
	
}

float VTciGrabber::FrameRate()
{
	float fRate;
	
	fRate = 1000*m_nFramesGrabbed/(float)m_timer.Time();
	if ( m_nFramesGrabbed == (ULONG_MAX-1)) {
		m_timer.Start();
		m_nFramesGrabbed = 0;
	}
	return fRate;
}