/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

///////////////////////////////////////////////////////////////////////////
// Filename	: TciGrabber.h
// Created	: Tuesday, June 17th, 1997
// Description: A class to encapsulate the functionality of the Coreco TCI
//					board.
// 
//  Vasanth Philomin 1997, vasi@cs.umd.edu
//  Nuria Oliver 1997, nuria@media.mit.edu
///////////////////////////////////////////////////////////////////////////
// TciGrabber.h : header file
//
#ifndef _TCIGRABBER_H
#define _TCIGRABBER_H

//#define _EVENTSUPPORT
#define HOST_FRAME_BUFFER	1
#define VGA_FRAME_BUFFER	0

#include "OdxIsr.h"
#include "ImgWnd.h"
#include "StaticFrame.h"
#include <afxmt.h>
#include "UErrors.h"
#include "UBitMap.h"
#include "VTimer.h"

static const	ErrorNumber				kErrNotGrabbing			= 1925;

class VTciGrabber;

struct ThreadParams	{
	VTciGrabber *grabber;
	BOOL		display;
	CDisplay	*disp;
	CListBox	*lbMsgs;
	HWND		hParentWnd;
};

class VTciGrabber	{
	friend UINT GrabThread(LPVOID param);
public:
	ErrorNumber DirtyReduce(TBitMap &dest, int factor = 2);
	ErrorNumber StartGrab(HWND hWnd, CListBox *lstBox, CDisplay *disp, BOOL display);
	void StopGrab();
	void Snap(CDisplay *disp);
	int Width()		{ return m_nWidth; }
	int Height()	{ return m_nHeight; }
	int PixelSize()	{ return m_nPixSize; }
	BOOL Grabbing()	{ return m_bGrabbing; }
	float	FrameRate(); 
	void Dispose();
	BOOL Init(CStringList *msgs = 0, int res_factor = 2);
	VTciGrabber();
	~VTciGrabber();
	
	
protected:
	HGLOBAL hBuf;
	HANDLE  m_Thread;
	ThreadParams params;
	BOOL m_bGrabbing;
	LPVOID m_lpBits;
	CEvent stopEvent;
	int m_nPixSize;
	int m_nWidth;
	int m_nHeight;
	void * ImgBuf2;
	void * ImgBuf1;
	CVTimer m_timer;
	int		m_nFramesGrabbed;
};

#endif
