// File       : UAdaptEM.h
// Creation   : Feb 1997
// Creator    : N. OLIVER
// Goal       : Adaptive Mixture of Gaussians learning algorithms
//
// Tab        : no

#ifndef _UADAPTEM_H
#define _UADAPTEM_H

// project includes
#include "ULearn.h"
#include "UStatistics.h"
#include "UHistogram.h"
#include "UBitMap.h"
#include "UErrors.h"
#include "UTimer.h"

#ifndef GenGAUSSTYPE
#define GenGAUSSTYPE double
typedef qmGaussian<GenGAUSSTYPE> TGenGaussian;
#endif

// Define the errors
static const ErrorNumber  kErrInitModels    =   1970;
static const ErrorNumber  kErrNoData        =   1971;


// It is thought for 2 dimensional data

class TAdaptEM : public TLearn
{
  public:

  ErrorNumber     Init (TMixGaussMutex  *m,
			TBitMap         *source,
			TROIMutex       *roi,
			TFlagMutex      *flag,
			UINT           maxNumComp = 1,
			TGenGaussian    *models = (TGenGaussian*)NULL,
			UINT           numComp = 0, 
			float           *weights = (float *)NULL,
			float            initDist = 0,
			UINT           dim = 2,
			BOOL          interior = true,
			float*           margin = (float*)NULL,
			float            updateTime = 5.,
			double           Tc = .9,
			double           initVar = 1.0,
			int              numIter = 1,
			char*            fName = (char*)NULL );
  // creates the learning structures for using EM learning
  // the number of components is <numComp>
  // initial statistics for the components can be provided in models
  // <Tc> is the threshold for creating new components

    void            Dispose ();
  // kill the object

  inline void       setNumIter( UINT n ) { kNumIter=n; }
  inline void       setMaxTerms( UINT m ) { kMaxTerms=m; }
  inline void       setMargin( float* m ) { m_arrMargin[0]=m[0];m_arrMargin[1]=m[1]; }
  ErrorNumber       Learn( );
  // learn the model of the region in source delimited by bb

  protected:
  double           m_fInitVar;         // initial variance 
  double*          m_pW;               // weights for each term of the model
  double*          m_pInitW;           // weights for the init models
  double*          m_pGaussOutputs;    // outputs for each term of the model
  double*          m_pScaledOutputs;   // scaled outputs for each term of the model
  double           m_fOutput;          // outputs for each term of the model
  double*          m_pAverageDist;     // average distance in the models
  float            m_fMaxDist;
  float            m_fLastMaxDist;
  BOOL          m_bUpdateML;
  float            m_fDensity;         // density of the object learned
  float            m_fFactorProb;
  float            m_fInitFactorProb;  // factor for the initial likelihoods
  double           m_fInitAverageLike; // average likelihood to prior
  double           m_fAverageLike;     // average likelihood
  double           m_fInitMaxDist;     // average distance in the models
  double           m_fInitNormLike;    // average distance in the models
  BOOL          m_bInitModelsFlag;  // flag to indicate whether the 
  // priors or the learnt models are active
  UINT           m_nNumTerms;        // number of terms in the model
  UINT           m_nInitNumTerms;    // init number of terms in the model
  TGenGaussian*    m_pModels;          // each of the gaussians
  TGenGaussian*    m_pInitModels;      // each of the init gaussians
  UINT           m_nNumIter;         // number of iterations
  UINT           m_nMaxObs;          // maximum number of observations
  UINT           m_nMaxTerms;        // maximum number of terms in the mixture
  double           m_fTc;              // threshold for adding new member
  UINT           m_nDim;
  BOOL          m_bInterior;        // compute interior or exterior of bb
  float            m_arrMargin[2];       // how much of the bb to process

  double*          m_pInputData;       // array with input data
  double*          m_pInput;
  UINT           m_nNumPts;          // number of points included in the model
  UINT           m_nNumObs;          // number of observations
  TTimer          *m_pTimer;           // timer to determine how often to learn
  float            m_fUpdateTime;      // update gap time
  float            m_fTimeElapsed;
  BOOL          m_bFirstTime;

  inline
  ErrorNumber TAdaptEM::InitProcess ( int&        oWidth,
				      int&        oHeight,
				      UINT&     oOffset,
				      UINT*      &oBase, 
				      CRect       &oBB,
				      float       &oDens );

  ErrorNumber TAdaptEM::ReadData();
  ErrorNumber TAdaptEM::GetGaussianOutputs();
  ErrorNumber TAdaptEM::AddTerm();
  ErrorNumber TAdaptEM::UpdateW( int    term,
				 double amt);
  ErrorNumber TAdaptEM::UpdateMean( int    term,
				    double amt);
  ErrorNumber TAdaptEM::UpdateCov(int    term,
				  double amt);
  ErrorNumber TAdaptEM::LearnStep();
  ErrorNumber TAdaptEM::Normalize();
  ErrorNumber TAdaptEM::FeedData();
};


#endif













