// File       : UAdaptEM.h
// Creation   : Feb 1997
// Creator    : N. OLIVER
// Goal       : Adaptive Mixture of Gaussians learning algorithms
//
// Tab        : no

#ifndef __AdaptEM__
#define __AdaptEM__

// project includes
#include "ULearn.h"
#include "qmGaussian.h++"
#include "UTypes.h"
#include "UHistogram.h"
#include "UBitMap.h"
#include "UErrors.h"
#include "UTimer.h"

#ifndef GenGAUSSTYPE
#define GenGAUSSTYPE double
typedef qmGaussian<GenGAUSSTYPE> TGenGaussian;
#endif

// It is thought for 2 dimensional data

class TAdaptEM : public TLearn
{
  public:

  ErrorNumber     Init (THistogram      *histo,
			TBitMap         *source,
			TLearnIndex     *ind,
			TROIMutex       *roi,
			UInt32           maxNumComp = 1,
			TGenGaussian    *models = (TGenGaussian*)NULL,
			UInt32           numComp = 0, 
			float           *weights = (float *)NULL,
			UInt16           dim = 2,
			double           initVar = 1.0,
			float            updateTime = 5.,
			//double           Tc = 0.60653066
			double           Tc = 0.2
			);
  // creates the learning structures for using EM learning
  // the number of components is <numComp>
  // initial statistics for the components can be provided in models
  // <Tc> is the threshold for creating new components

    void            Dispose ();
  // kill the object

  inline void       setNumIter( UInt32 n ) { kNumIter=n; }
  inline void       setMaxTerms( UInt32 m ) { kMaxTerms=m; }
  ErrorNumber       Learn( );
  // learn the model of the region in source delimited by bb

  protected:
  double           kInitVar;         // initial variance 
  double*          fw;               // weights for each term of the model
  double*          fGaussOutputs;    // outputs for each term of the model
  double*          fScaledOutputs;   // scaled outputs for each term of the model
  double           fOutput;          // outputs for each term of the model
  double*          fAverageDist;     // average distance in the models
  UInt32           fNumTerms;        // number of terms in the model
  TGenGaussian*    fModels;          // each of the gaussians
  UInt32           kNumIter;         // number of iterations
  UInt32           kMaxObs;          // maximum number of observations
  UInt32           kMaxTerms;        // maximum number of terms in the mixture
  double           kTc;              // threshold for adding new member

  double**         fInputData;       // array with input data
  double*          fInput;
  UInt32           fNumPts;          // number of points included in the model
  UInt32           fNumObs;          // number of observations
  TTimer          *fTimer;           // timer to determine how often to learn
  float            fUpdateTime;      // update gap time

  inline
  ErrorNumber TAdaptEM::InitProcess ( int&        oWidth,
				      int&        oHeight,
				      UInt32&     oOffset,
				      UInt8*      &oBase);

  ErrorNumber TAdaptEM::ReadData();
  ErrorNumber TAdaptEM::GetGaussianOutputs();
  ErrorNumber TAdaptEM::AddTerm();
  ErrorNumber TAdaptEM::UpdateW( int    term,
				 double amt);
  ErrorNumber TAdaptEM::UpdateMean( int    term,
				    double amt);
  ErrorNumber TAdaptEM::UpdateCov(int    term,
				  double amt);
  ErrorNumber TAdaptEM::LearnStep();
  ErrorNumber TAdaptEM::Normalize();
  ErrorNumber TAdaptEM::FeedData();

};


#endif













