/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// File       : UConnex.h
// Creation   : 07/1997
// Goal       : declares the TConnex class, a class for finding the biggest
//              connex class in a bitmap
//
// Tab        : no
//  Nuria Oliver 1997, nuria@media.mit.edu


#ifndef _UCONNEX_H
#define _UCONNEX_H

// project includes
#include "UErrors.h"
#include "UBitMap.h"
#include "UStatistics.h"


static const DWORD   kConnexDefaultColor = 0x000000ff; // (red in xBGR)


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//                        typedefs, consts & globals                          //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

typedef struct
{
	WORD   nIn;    // if mIn != index, the box is included in the box indexed
	//  by mIn
	DWORD  nArea;  // number of pixels in this class
} TConnexClass;





////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//                         TConnex class declaration                          //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class TConnex
{
public:
	
    ErrorNumber       Init ();
    void              Dispose ();
	
    ErrorNumber       Compute  (TBitMap*    iSource,
		int         iRadius,
								CRect*      iROI,
                                CRect*      iSupportRect,
                                LPVOID      iSupportColor,
                                CRect*      oBounding,
                                CRect*      oBoundingPos,
                                TBitMap*    oSupport,
                                TGaussianf* oGaussian,
                                BOOL&       oExists);
    // Find the biggest connex class in <iSource>. Pixel are supposed connex
    //    if their distance is less or equal to <iRadius>.
    // If <iROI> is not NULL, the search is restrected inside <iROI>.
    // If <oSupport> is not NULL, it will be setup to represent the
    //    biggest connex pixel area
    // If <iSupportRect> is not NULL, it defines the area in <oSupport> where
    //    to represent the support map, else the ValidRect of <oSupport> will
    //    be used.
    // If <iSupportColor> is not NULL, it points to the pixel value that will
    //    be used for displaying the support map in <oSupport>. Else a default
    //    value will be used.
    // If <oBounding> is not NULL, it will contain the bouding box.
    // If <oBoundingPos> is not NULL, it will decribe the position of the
    //    extremas : oBoundingPos->left will be the y position of one (if many)
    //    of the left extremas.
    // WARNING ! <oBoundingPos> will only be computed if <oBounding>
    //    is computed.
    // If <oGaussian> is not NULL, it will contain the spatial distribution.
    // If the is no white pixel in input bitmap, oExists = false, and no other
    //    output parameter is modified.
	
	
protected:
	
    LPDWORD         m_pCompoId;     // support map as integers
    TConnexClass   *m_pDomTab;      // table of classes
    LPVOID          m_pBuffer;
    DWORD           m_nSize;
	
    ErrorNumber    InitProcess (TBitMap*    iSource,
		int         iRadius,
		CRect*      iROI,
		DWORD&      oWidth,
		DWORD&      oHeight,
		DWORD&      oOffset,
		LPBYTE      &oBase);
    // initialise data members for processing
	
    void            Representant (DWORD& ioIndex);
    // replace an index value by its representant value
	
    DWORD          GetRepresentant (DWORD iIndex);
    // returns the representant of a class
	
    void            ProcessNeighboorPixel  (LPDWORD   neighPtr,
		LPDWORD   spPtr,
		DWORD&    currentClass);
    // process one neighboor pixel
	
    DWORD          FindBiggest (DWORD iNbClasses);
    // return index of the biggest
	
    ErrorNumber     GenerateOutPut (DWORD       classN,
		DWORD       iWidth,
		DWORD       iHeight,
		CRect*      oBounding,
		CRect*      oBoundingPos,
		TBitMap*    oSupport,
		CRect*      iSupportRect,
		LPVOID      iSupportColor,
		TGaussianf* oGaussian);
	
};


#endif
