/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// UMatrix3.h
//  Nuria Oliver 1997, nuria@media.mit.edu

#ifndef _UMATRIX3_H_
#define _UMATRIX3_H_

//Class for symmetrical matrixes

template<class T> class TVector2;

template< class T >
class TMatrix3
{
    friend class TVector2<T>;
private:
    T *m_pAllocArray;
	
public:
    TMatrix3(){m_pAllocArray = new T[3];}
    TMatrix3(T e1, T e2, T e3){m_pAllocArray = new T[3];set(e1, e2, e3);}
    TMatrix3( T* data ){m_pAllocArray = new T[3];set( data ); }
    TMatrix3( TMatrix3<T>& m ){m_pAllocArray = new T[3];set( m ); }
	
    ~TMatrix3() {if (m_pAllocArray) delete m_pAllocArray;}
	
	T* getData() { return m_pAllocArray; }
	
	T& operator()( int i ){return m_pAllocArray[i];}
	
	void set(T* v);
	void set(T e1, T e2, T e3);
	void set(TMatrix3<T>& m){set(m(0), m(1), m(2));}
	void setValue(T val);
	void setElem( int i , T val ) { m_pAllocArray[i]= val ; }
	
	void setZero() {setValue(T(0));}
	void setIdentity();
	void setAddEq(T e1, T e2, T e3);
	void setAddEq(T* v1 );
	void setAddEq(TMatrix3<T> &m ){ setAddEq( m.getData() );}
	void setAdd(T* v1, T* v2 );
	void setSubtractEq(T e1, T e2, T e3);
	void setSubtractEq(T* v1 );
	void setSubtract(T* v1, T* v2 );
	void setOuterProduct( T* v );
	void setAddOuterProduct( T* v );
	void setOuterProduct( T e1, T e2 );
	void setOuterProduct( TVector2<T>& v ){ setOuterProduct( v(0), v(1) ); }
	void setAddOuterProduct( T e1, T e2 );
	void setAddOuterProduct( TVector2<T>& v ){setAddOuterProduct(v(0), v(1));}
	void setSubtractOuterProduct( T* v );
	void setSubtractOuterProduct( TVector2<T>& v );
	void setSubtractOuterProduct( T e1, T e2 );
	void setDivideEq( float val );
	void setScaleEq( float val );
	void setStringValue( char* s );
	
	int isSingular() { return ( getDet() == 0 ); }
    
	T getDet();
	int getInvert(TMatrix3<T>&);
	int setInvert(TMatrix3<T>&);
	int setInvertEq();
    
	void operator*=(  T val ) { setScaleEq( val ); }
	void operator/=(  T val ) { setDivideEq( val ); }
	TMatrix3& operator=(  TMatrix3<T>& val ) { set( val ); return *this; }
	int  operator==( const TMatrix3<T>& );
	
	void	Write( CFile	&f );
};

template<class T>
void TMatrix3<T>::set( T* v )
{
    m_pAllocArray[0]=v[0];
    m_pAllocArray[1]=v[1];
    m_pAllocArray[2]=v[2];
}

template<class T>
void TMatrix3<T>::set( T e1, T e2, T e3 )
{
    m_pAllocArray[0]=e1;
    m_pAllocArray[1]=e2;
    m_pAllocArray[2]=e3;
}

template<class T> 
void TMatrix3<T>:: setValue( T val )
{
    set( val, val, val );
}

template<class T>
void TMatrix3<T>:: setIdentity()
{
    m_pAllocArray[0]=m_pAllocArray[2]=1.0;
    m_pAllocArray[1]=0.0;
}

template<class T>
void TMatrix3<T>:: setAddEq( T* v )
{
    m_pAllocArray[0]+=v[0];
    m_pAllocArray[1]+=v[1];
    m_pAllocArray[2]+=v[2];
}

template<class T>
void TMatrix3<T>:: setAddEq( T e1, T e2, T e3 )
{
    m_pAllocArray[0]+=e1;
    m_pAllocArray[1]+=e2;
    m_pAllocArray[2]+=e3;
}

template<class T>
void TMatrix3<T>:: setAdd( T* v1, T* v2 )
{
    m_pAllocArray[0]=v1[0]+v2[0];
    m_pAllocArray[1]=v1[1]+v2[1];
    m_pAllocArray[2]=v1[2]+v2[2];
}


template<class T>
void TMatrix3<T>:: setSubtractEq( T* v )
{
    m_pAllocArray[0]-=v[0];
    m_pAllocArray[1]-=v[1];
    m_pAllocArray[2]-=v[2];
}

template<class T>
void TMatrix3<T>:: setSubtractEq( T e1, T e2, T e3 )
{
    m_pAllocArray[0]-=e1;
    m_pAllocArray[1]-=e2;
    m_pAllocArray[2]-=e3;
}

template<class T>
void TMatrix3<T>:: setSubtract( T* v1, T* v2 )
{
    m_pAllocArray[0]=v1[0]-v2[0];
    m_pAllocArray[1]=v1[1]-v2[1];
    m_pAllocArray[2]=v1[2]-v2[2];
}

template<class T> 
void TMatrix3<T>:: setOuterProduct( T* v )
{
    m_pAllocArray[0]=v[0]*v[0];
    m_pAllocArray[1]=v[0]*v[1];
    m_pAllocArray[2]=v[1]*v[1];
}

template<class T> 
void TMatrix3<T>:: setOuterProduct( T e1, T e2 )
{
    m_pAllocArray[0]=e1*e1;
    m_pAllocArray[1]=e1*e2;
    m_pAllocArray[2]=e2*e2;
}

template<class T> 
void TMatrix3<T>:: setAddOuterProduct( T* v )
{
    m_pAllocArray[0]+=v[0]*v[0];
    m_pAllocArray[1]+=v[0]*v[1];
    m_pAllocArray[2]+=v[1]*v[1];
}

template<class T> 
void TMatrix3<T>:: setAddOuterProduct( T e1, T e2 )
{
    m_pAllocArray[0]+=e1*e1;
    m_pAllocArray[1]+=e1*e2;
    m_pAllocArray[2]+=e2*e2;
}


template<class T> 
void TMatrix3<T>:: setSubtractOuterProduct( T* v )
{
    m_pAllocArray[0]-=v[0]*v[0];
    m_pAllocArray[1]-=v[0]*v[1];
    m_pAllocArray[2]-=v[1]*v[1];
}

template<class T> 
void TMatrix3<T>:: setSubtractOuterProduct( T e1, T e2 )
{
    m_pAllocArray[0]-=e1*e1;
    m_pAllocArray[1]-=e1*e2;
    m_pAllocArray[2]-=e2*e2;
}

template<class T> 
void TMatrix3<T>:: setSubtractOuterProduct( TVector2<T>& v )
{
    m_pAllocArray[0]-=v(0)*v(0);
    m_pAllocArray[1]-=v(0)*v(1);
    m_pAllocArray[2]-=v(1)*v(1);
}

template<class T>
void TMatrix3<T>:: setDivideEq( float val )
{
    m_pAllocArray[0]/=val;
    m_pAllocArray[1]/=val;
    m_pAllocArray[2]/=val;
}

template<class T>
void TMatrix3<T>:: setScaleEq( float val )
{
    m_pAllocArray[0]*=val;
    m_pAllocArray[1]*=val;
    m_pAllocArray[2]*=val;
}

template<class T> 
T TMatrix3<T>::getDet()
{
    return( m_pAllocArray[0]*m_pAllocArray[2]-m_pAllocArray[1]*m_pAllocArray[1] );
}

template<class T> 
int TMatrix3<T>::setInvert( TMatrix3<T> &m )
{
    TMatrix3 tmp( m.getData() );
    T det = m.getDet();
    if ( det == 0 )
		return 1;
    else {
		m_pAllocArray[0]=tmp(2)/det;
		m_pAllocArray[1]=-tmp(1)/det;
		m_pAllocArray[2]=tmp(0)/det;
		return 0;
    }
}


template<class T> 
int TMatrix3<T>::getInvert( TMatrix3<T> &m )
{
    T det = getDet();
    if ( det == 0 ) {
		m.setIdentity();
		return 1;
    } else {
		m.set( m_pAllocArray[2]/det, -m_pAllocArray[1]/det, m_pAllocArray[0]/det );
		return 0;
    }
}

template<class T>
int TMatrix3<T>::setInvertEq()
{
    return setInvert( *this );
}

template<class T>
int
TMatrix3<T>::
operator==( const TMatrix3<T>& m ) 
{
    for ( register int i=0; i<3; i++ )
		if ( m_pAllocArray[i]!=m(i) ) return 0;
		return 1;
}

template<class T>
void
TMatrix3<T>::Write( CFile	&f )
{
	CString		buf;
	buf.Format("%f %f %f ", m_pAllocArray[0], m_pAllocArray[1], m_pAllocArray[2]);
	f.Write( buf, buf.GetLength() );
}

#endif /* _UMATRIX3_H_ */
